@@ -26,6 +26,7 @@
 static int index[SNDRV_CARDS] = SNDRV_DEFAULT_IDX;
 static char *id[SNDRV_CARDS] = SNDRV_DEFAULT_STR;
 static int enable[SNDRV_CARDS] = SNDRV_DEFAULT_ENABLE_PNP;
+static int pcifix[SNDRV_CARDS] = { [0 ... (SNDRV_CARDS-1)] = 255 };
 
 MODULE_PARM(index, "1-" __MODULE_STRING(SNDRV_CARDS) "i");
 MODULE_PARM_DESC(index, "Index value for " CARD_NAME " soundcard.");
@@ -36,12 +37,22 @@ MODULE_PARM_SYNTAX(id, SNDRV_ID_DESC);
 MODULE_PARM(enable, "1-" __MODULE_STRING(SNDRV_CARDS) "i");
 MODULE_PARM_DESC(enable, "Enable " CARD_NAME " soundcard.");
 MODULE_PARM_SYNTAX(enable, SNDRV_ENABLE_DESC);
+MODULE_PARM(pcifix, "1-255i");
+MODULE_PARM_DESC(pcifix, "Enable VIA-workaround for " CARD_NAME " soundcard.");
+MODULE_PARM_SYNTAX(pcifix, SNDRV_ENABLED ",allows:{{0,Disabled},{1,Latency},{2,Bridge},{3,Both},{255,Auto}},default:4,dialog:check");
 
 MODULE_DESCRIPTION("Aureal vortex");
 MODULE_CLASSES("{sound}");
 MODULE_LICENSE("GPL");
 MODULE_DEVICES("{{Aureal Semiconductor Inc., Aureal Vortex Sound Processor}}");
 
+#ifndef PCI_VENDOR_ID_VIA 
+#define PCI_VENDOR_ID_VIA 0x1106
+#endif
+#ifndef PCI_DEVICE_ID_VIA_8365_1
+#define PCI_DEVICE_ID_VIA_8365_1 0x8305
+#endif
+
 #ifndef MODULE
 /* format is: snd-mychip=enable,index,id */
 static int __init alsa_card_vortex_setup(char *str) {
@@ -61,6 +72,59 @@ __setup("snd-au88x0=", alsa_card_vortex_setup);
 
 MODULE_DEVICE_TABLE(pci, snd_vortex_ids);
 
+static void __devinit snd_vortex_workaround(struct pci_dev *vortex, int fix)
+{
+	int rc;
+	struct pci_dev *via = NULL;
+
+		/* autodetect if workarounds are required */
+	if ((via = pci_find_device(PCI_VENDOR_ID_VIA, PCI_DEVICE_ID_VIA_8365_1, NULL))) {
+		if(fix == 255) {
+			printk(KERN_INFO CARD_NAME
+				": detected VIA KT133/KM133. activating workaround...\n");
+			fix = 3; // do latency and via bridge workaround
+		}
+	}
+
+		/* default value - nothing to do */
+	if (fix == 255)
+		return;
+	
+		/* fix vortex latency */
+	if(fix & 0x01) {
+		if( !(rc = pci_write_config_byte(vortex, 0x40, 0xff)) ) {
+			printk(KERN_INFO CARD_NAME
+					": vortex latency is 0xff\n");
+		}
+		else {
+			printk(KERN_WARNING CARD_NAME ": could not set vortex latency: pci error 0x%x\n", rc);
+		}
+	}
+
+		/* fix via agp bridge */
+	if(via && (fix & 0x02)) {
+		u8 value;
+
+			/*
+			 * only set the bit (Extend PCI#2 Internal Master for
+			 * Efficient Handling of Dummy Requests) if the can
+			 * read the config and it is not already set
+			 */
+
+		if( !(rc = pci_read_config_byte(via, 0x42, &value)) && (
+			(value & 0x10) ||
+			!(rc=pci_write_config_byte(via, 0x42, value|0x10)) ) ) {
+
+			printk(KERN_INFO CARD_NAME
+					": bridge config is 0x%x\n",
+					value|0x10);
+		}
+		else {
+			printk(KERN_WARNING CARD_NAME ": could not set vortex latency: pci error 0x%x\n", rc);
+		}
+	}
+}
+
 
 // component-destructor
 // (see "Management of Cards and Components")
@@ -95,11 +159,11 @@ snd_vortex_create(snd_card_t *card, struct pci_dev *pci, vortex_t **rchip) {
     // check PCI availability (DMA).
     if ((err = pci_enable_device(pci)) < 0)
         return err;
-    if (!pci_dma_supported(pci, VORTEX_DMA_MASK)) {
+   if (pci_set_dma_mask(pci, VORTEX_DMA_MASK) < 0 ||
+       pci_set_consistent_dma_mask(pci, VORTEX_DMA_MASK) << 0) {
         printk(KERN_ERR "error to set DMA mask\n");
         return -ENXIO;
     }
-    pci_set_dma_mask(pci, VORTEX_DMA_MASK);
 
     chip = snd_magic_kcalloc(vortex_t, 0, GFP_KERNEL);
     if (chip == NULL)
@@ -153,6 +217,8 @@ snd_vortex_create(snd_card_t *card, struct pci_dev *pci, vortex_t **rchip) {
     	goto alloc_out;
     }
 
+    snd_card_set_dev(card, &pci->dev);
+
     *rchip = chip;
 
     return 0;
@@ -199,6 +265,7 @@ snd_vortex_probe(struct pci_dev *pci, const struct pci_device_id *pci_id) {
         snd_card_free(card);
         return err;
     }
+    snd_vortex_workaround(pci, pcifix[dev]);
     // (4) Alloc components.
     // ADB pcm.
     if ((err = snd_vortex_new_pcm(chip, VORTEX_PCM_ADB, NR_ADB)) < 0) {
